/*  $Id: Sequencer.java 1335 2012-08-25 13:27:49Z sdevaux $
 *
 * Tabs = 4 chars
 * copyright (C) 2002-2012 Sebastien Devaux
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 *              Sebastien Devaux <sebastien.devaux@laposte.net>
 */
package net.eduvax.util;

import java.util.Hashtable;
	/** 
	 * Permet le lancement séquentiel d'action 
     * dans un thread propre du sequenceur.
     *  
     * <p>Utilisations possibles : 
     * - dialogue avec des API dont le contexte est déterminé par le thread 
     * courant.
     * - dialogue avec API non ou partiellement réentrante (ex : API OpalRT 
     * qui n'aime pas qu'un même noeud soit piloté par plusieurs threads).</p>
     * @djin.stereotype thread
     * @djin.link use,EThread
     */ 
    public class Sequencer implements Runnable
    {
        /** Thread propre du sequenceur. */
        private EThread _thread;
        /** 
         * Construit un nouveau séquenceur et démarre son thread.
         *  
         * @param nom nom du sequenceur, le thread du sequenceur sera
         * baptisé selon ce nom
         */ 
        public Sequencer(String name)
        {
            _nextToRun=null;
            _run=true;
			_runStatus=new Hashtable<Runnable,Boolean>();
            _errHandler=App.get().getErrHandler();
            _thread=new EThread(""+getClass()+"/"+name,this);
            _thread.start();
        }
        
        /** 
         * Demander l'arrêt du séquenceur.
         *  
         * <p>L'arrêt est effectif à la fin de la commande en cours s'il y en 
         * a une.</p>
         */ 
        public void stop()
        {
            synchronized(this) {
            	_run=false;
            	EThread.doNotifyAll(this);
            }
        }
        /** Obtenir le thread du sequenceur. 
         * <p>Attention, ce n'est pas pour faire n'importe quoi !
         * En principe on devrait se limiter à des choses relativement neutres
         * comme getName() ou Join().</p>
         * @return Thread du sequenceur.
         */ 
        public EThread getThread()
        {
            return _thread;
        }
        
        /** 
         * Demander l'execution d'un Runnable au thread du séquenceur.
         *  
         * @param toRun Runnable à confier au sequenceur
         * <p>Cet appel bloque jusqu'à ce que la commande soit traitée</p>
         */
		public void run(Runnable toRun) {
			run(toRun,0);
		}
        /** 
         * Demander l'execution d'un Runnable au thread du séquenceur.
         *  
         * <p>Cet appel bloque jusqu'à ce que la commande soit traitée ou
		 * que le delai timeout soit expiré.</para>
         * @param toRun Runnable à confier au sequenceur
         * @param timeout Délai d'attente maximal (millisecondes)
         * 
		 * @return 
         * - true : la commande a été traité
         * - false si elle n'a pas commencé ou terminé avant expiration du 
         * délai. C'est le cas en particulier si le sequenceur est déjà
         * arrêté.
         */ 
		public boolean run(Runnable toRun,int timeout)
		{
            boolean timeouted=false;
			// Attendre que le thread Opal du noeud soit libre.
			synchronized(this) {
				while (_nextToRun!=null && _run)
				{
					try {
						timeouted=!EThread.doWait(this,timeout);
					} catch (InterruptedException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}
				if (!timeouted)
				{
					_nextToRun=toRun;
					_runStatus.put(toRun, false);
				}                 
             // Attention dans la suite, il est important et
             // volontaire de prendre toRun et non _nextToRun pour continuer
             // de travailler. En effet on attendra la terminaison de la
             // commande reçue en paramètre et non d'une autre qui aurait
             // pu être mise en attente dans l'intervale.
				EThread.doNotify(this);
			}
            if (!timeouted)
            {
                // Attendre que le thread du sequencer ait fini de traiter la
                // commande. 
                synchronized(this) {
                	while(!_runStatus.get(toRun)&&!timeouted&&_run)
                	{
                		try {
							timeouted=!EThread.doWait(this,timeout);
						} catch (InterruptedException e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}
                	}
                	_runStatus.remove(toRun);
                }
            }
            return !timeouted;
		}
        /** 
         * point d'entrée du thread du séquenceur
         *  
         * <p>Ne pas appeler cette méthode, il s'agit du callback
         * pour le thread qui pilote le sequenceur. Ce thread est démarré
         * directement par le final staticructeur.</p>
         */ 
        public void run()
        {
            boolean run=_run;
			while (run)
			{
				Runnable toRun;
				// Attendre l'arrive d'une commande
				// ou le signal d'arret.
				synchronized (this) {
					while (_nextToRun==null && _run) {
						try {
							EThread.doWait(this);
						} catch (InterruptedException e) {
							e.printStackTrace(); // TODO: something better
						}
					}
					run=_run;
					toRun=_nextToRun;
				}
				// si une commande est arrivée, on la lance. 
				if (toRun!=null) {
                    try {
    					toRun.run();
                    }
                    catch (Exception ex)
                    {
                        // On capture toute les exceptions car planter le 
                        // thread du sequencer est potentiellement plus
                        // dangereux que de continuer.
                        // on commance par arrêter le sequenceur pour limiter
                        // les dégats.
                        stop();
                        Error err=
                            new Error(""+getClass()+".exc",Error.ELevel.ERR,ex);
                        err.addArg(toRun);
                        _errHandler.handle(err);
                    }
                    // signaler aux eventuels appelant en attente que
                    // le thread est dispo pour une autre commande.
                    synchronized (this) {
                    	_nextToRun=null;
                    	if (_runStatus.get(toRun)!=null)
                    	{
                    		_runStatus.put(toRun,true); 
                    	}
                    	EThread.doNotify(this);
                    }
				}
			}
        }

        /** Etat d'activité du séquenceur */
        private boolean _run;
        /** Prochaine commande à executer par le séquenceur 
         * @djin.relation aggregation
         */ 
        private Runnable _nextToRun;
		/** Status des commande en attente et en cours */
		Hashtable<Runnable,Boolean> _runStatus;
        /** Gestionnaire d'erreur */
        private ErrorHandler _errHandler;
    }

