/*  $Id: StringTokenizer.java 1035 2010-06-15 13:08:33Z sdevaux $
 *
 * Tabs = 4 chars
 * copyright (C) 2002 Sebastien Devaux
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 *              Sebastien Devaux <sebastien.devaux@laposte.net>
 */
package net.eduvax.util;

import java.util.NoSuchElementException;
import java.util.Vector;

/**
 * Do the quite same thing than java.util.StringTokenizer. 
 * The difference is that tokenization do not operate inside
 * quotes. Exemple : <code>this is "a test" string</code>
 * will be tokenized as :
 * <pre>this
 * is
 * a test
 * string</pre>
 * @version $Id: StringTokenizer.java 1035 2010-06-15 13:08:33Z sdevaux $
 * @author Sebastien Devaux
 */
public class StringTokenizer extends java.util.StringTokenizer {
	/**
	 * Create new tokenizer using default separators.
	 * Default separator are " \r\n\t\f"
	 * @param str string to tokenize.
	 */ 
	public StringTokenizer(String str) {
		this(str,DEF_DELIMITERS);
	}
	/**
	 * Create new tokenizer
	 * @param str string to tokenize.
	 * @param delim tokenization delimiters.
	 */
	public StringTokenizer(String str,String delim) {
		super(str,delim);
		_index=0;
		_delimiters=delim;
		_toSplit=str;
	}
	/**
	 * @return true if there are more tokens, false if all tokens 
	 * have been already returned
	 */
	public boolean hasMoreTokens() {
		return _index<_toSplit.length();
	}
	public boolean hasMoreElements() {
		return hasMoreTokens();
	}
	/**
	 * @return next token if any
	 */
	public String nextToken() throws NoSuchElementException {
		StringBuffer res=new StringBuffer();
		boolean token=false;
		boolean skip=false;
		if (_index>=_toSplit.length()) {
			throw new NoSuchElementException();
		}
		while (_index<_toSplit.length() && !token) {
			char ch=_toSplit.charAt(_index);
			_index++;
			if (ch==QUOTE) {
				skip=!skip;
			}
			else if (!skip&&_delimiters.indexOf(ch)>=0) {
				token=res.length()>0;
			}
			else {
				res.append(ch);
			}
		}
		return res.toString();
	}
	public Object nextElement() {
		return nextToken();
	}
	/**
	 * Returns all remaining tokens as String array.
	 * @return tokens as String array.
	 */ 
	public String[] split() {
		Vector<String> resVect=new Vector<String>();
		while (hasMoreTokens()) {
			resVect.add(nextToken());
		}
		String[] res=new String[resVect.size()];
		return resVect.toArray(res);
	}
	/** current index in string tokenization. */
	private int _index;
	/** separation chars. */
	private String _delimiters;
	/** string to be splited from tokens. */
	private String _toSplit;
	/** quote char */
	private final static char QUOTE='"';
	/** defaults tokens delimiters */
	private final static String DEF_DELIMITERS=" \t\n\r\f";
}
